/*
  host_map.cc

  $Id: host_map.c,v 1.1 1999/11/27 02:59:30 everton Exp $
 */

#include <string.h>
#include <syslog.h>
#include <errno.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "portfwd.h"
#include "host_map.hpp"
#include "solve.h"

void host_map::show() const
{
  iterator<vector<from_addr*>,from_addr*> it(*src_list);
  it.start();
  if (it.cont()) {
    it.get()->show();
    it.next();
  }
  for (; it.cont(); it.next()) {
    syslog(LOG_INFO, ", ");
    it.get()->show();
  }
  syslog(LOG_INFO, " => ");
  dst_addr->show(); 
}

/*
 * Returns -1 on failure; file descriptor on success.
 */
int host_map::pipe(const struct ip_addr *ip, int port) const
{
  const struct ip_addr *dst_ip  = dst_addr->get_addr();
  int                  dst_port = dst_addr->get_port();

  ONVERBOSE(syslog(LOG_DEBUG, "TCP pipe: %s:%d => %s:%d", addrtostr(ip), port, addrtostr(dst_ip), dst_port));

  int rsd = socket(AF_INET, SOCK_STREAM, get_protonumber(P_TCP));
  if (rsd == -1) {
    syslog(LOG_ERR, "pipe: Can't create TCP socket: %s\n", strerror(errno));
    return -1;
  }

  struct sockaddr_in sa;
  sa.sin_family      = AF_INET;
  sa.sin_port        = htons(dst_port);
  sa.sin_addr.s_addr = *((unsigned int *) dst_ip->addr);
  memset((char *) sa.sin_zero, 0, sizeof(sa.sin_zero));

  if (connect(rsd, (struct sockaddr *) &sa, sizeof(sa))) {
    syslog(LOG_ERR, "pipe: Can't connect to %s:%d: %s\n", inet_ntoa(sa.sin_addr), port, strerror(errno));
    close(rsd);
    return -1;
  }

  return rsd;
}

/*
 * Returns -1 on failure; file descriptor on success.
 */
int host_map::tcp_match(const struct ip_addr *ip, int port) const
{
  iterator<vector<from_addr*>,from_addr*> it(*src_list);
  for (it.start(); it.cont(); it.next())
    if (it.get()->match(ip, port))
      return pipe(ip, port);

  return -1;
}

void host_map::forward(const struct ip_addr *ip, int port, const char *buf, int buf_len) const
{
  const struct ip_addr *dst_ip  = dst_addr->get_addr();
  int                  dst_port = dst_addr->get_port();

  ONVERBOSE(syslog(LOG_DEBUG, "UDP forward: %s:%d => %s:%d", addrtostr(ip), port, addrtostr(dst_ip), dst_port));

  int rsd = socket(AF_INET, SOCK_DGRAM, get_protonumber(P_UDP));
  if (rsd == -1) {
    syslog(LOG_ERR, "forward: Can't create UDP socket: %s\n", strerror(errno));
    return;
  }

  struct sockaddr_in sa;
  sa.sin_family      = AF_INET;
  sa.sin_port        = htons(dst_port);
  sa.sin_addr.s_addr = *((unsigned int *) dst_ip->addr);
  memset((char *) sa.sin_zero, 0, sizeof(sa.sin_zero));

  int wr = sendto(rsd, buf, buf_len, 0, (struct sockaddr *) &sa, sizeof(sa));
  if (wr < 0)
    syslog(LOG_ERR, "forward: sendto failed: %m");
  else if (wr < buf_len)
    syslog(LOG_ERR, "forward: Partial write on sendto: %m");

  if (close(rsd))
    syslog(LOG_WARNING, "forward: close on socket failed: %m");
}

int host_map::udp_match(const struct ip_addr *ip, int port, const char *buf, int buf_len) const
{
  iterator<vector<from_addr*>,from_addr*> it(*src_list);
  for (it.start(); it.cont(); it.next())
    if (it.get()->match(ip, port)) {
      forward(ip, port, buf, buf_len);
      return -1;
    }

  return 0;
}

/* Eof: host_map.cc */
